#!/tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl
use strict;
use PureMessage::Manager::Request qw($REQUEST %in);
use PureMessage::Manager::Response::ServerGroups;
use PureMessage::Manager::Lang;
use PureMessage::ServerGroup qw(:all);
use PureMessage::ServerGroup::Publication;
use PureMessage::Version qw($PREFIX);
my %vars;
our %text;

my $r = $REQUEST->get_response( 'PureMessage::Manager::Response::ServerGroups',
                                'stderr_to_stdout' =>1
                              );

$r->add_help(text => "Help", tab => "servgrp", page => "mgr_servgrp_pub" );

$REQUEST->typedecl("name", "#nospaces #hb_as_option #value", "Publication Name");
my @v_errors = $REQUEST->validate();

my $id = $REQUEST->query('id');
my $new = $REQUEST->query('name');
my $p;

if ( @v_errors ) {
    $r->raise_error($v_errors[0]);
    $new = undef;
}

if ( $id || $new ) {
    $r->title(text('publ_title', $id || $new));
}
else {
    $r->title(text('publ_create_title'));
}

my $h = get_hosts();
unless (keys %{$h}) {
    $r->print_notification("There are no hosts found in your Server Group configuration.");
}

$p = PureMessage::ServerGroup::Publication->new($id || $new , create=> $new ? 1 : 0);
if ($id and not $p) {
    $r->raise_error("Publication $id doesn't exist.");
    $r->send();
    exit;
}
if ($new and not $p) {
    $r->raise_error("Can't create publication $new.");
    $r->send();
    exit;
}

$id = $p->{name} if $p;
if ( $REQUEST->query('add_host') ) {
    $p->subscribe($REQUEST->query('add_host'));
    if ( my $err_msg = $p->error ) {
        $r->raise_error($err_msg);
        $p->{error} = "";
    }
}
if ($new) {
    process_actions();
}

main_form();

sub process_actions {
    my $WIDGET = new PureMessage::Manager::Widget;
    $WIDGET->read_input($REQUEST, 'edit_publ');
    my $q = $WIDGET->result_data();
    
    if ( $q->{delete} ) {
        $p->delete();
        $r->dialog( style => 'dlg_info',
                    msg => &text("group_edited"),
                    buttons => [[ { type=>'image', icon=>'ok',text=>'Continue', href=>'index.cgi'} ]]
                  );
        $r->send;
        exit;
    }
    elsif ( $p and $REQUEST->query('save') or $q->{save}) {
        if ( $REQUEST->query('name') ne $p->{name} ) {
            $p->rename($REQUEST->query('name'));
            $id = $REQUEST->query('name');
        }
        if ( $p->{description} ne $REQUEST->query('description') ) {
            $p->attrib('description', $REQUEST->query('description'));
        }
    }
}

sub main_form {
    my @btns = ({ type=>'icon',icon=>'save',text=>'Save', name=>'save'});
    push @btns, { type=>'icon',icon=>'delete',text=>'Delete', name=>'delete'}
        if $REQUEST->query('id');
    
    my $tb = $r->ToolBar( name => 'tb',
                          buttons => [\@btns],
                          width=>500,
                        );
    
    my ( $fields, $data ) = get_form_data();
    my $edit_ssl = $r->addForm (
                                    name => "edit_publ",
                                    data => $data,
                                    fields => $fields,
                                    columns => 1,
                                    width => 500,
                                    title => text('properties'),
                                    toolbar => $tb,
                                    action => 'publication.cgi',
                                );
    
    if ( $id ) {
        $r->{objects_ui} = get_objects_ui();
        if ( get_objects() ) {
            $r->add_part($r->{objects_ui});
        }
        else {
            $r->addGrid(
                name => 'no_files',
                title=> "There are no shared files, lists or maps in this publication.",
                width=>500,
                );   
        }
        $r->{hosts_ui} = get_hosts_ui();
        if ( get_subscribers() ) {
            $r->add_part($r->{hosts_ui});
        }
        else {
            $r->addGrid(
                name => 'hosts_grid',
                title=> "This publication doesn't have any subscribers.",
                toolbar => \&hosts_tb,
                width=>500,
                action=>"publication.cgi?id=$id",
                );   
        }
    }
}

$r->send();

sub get_form_data {
    my $fields =  [ text('publ_name'), text('description') ];
    my %ui = ( 
        text('publ_name') => "<input type=hidden name='id' value='$id'>"
                           . "<input class=stretch value='$p->{name}' name='name'>",
        text('description') => "<textarea cols=30 rows=2 name='description' class=stretch>".$p->{description}."</textarea>"
             );
    if ( $p->{created} ) {
        push @$fields, text('created');
        $ui{text('created')} = $p->{created};
        $ui{text('created')} =~ s/T/ /;
    }
    if ( $p->{modified} ) {
        push @$fields, text('modified');
        $ui{text('modified')} = $p->{modified};
        $ui{text('modified')} =~ s/T/ /;
    }
    return ($fields, \%ui);

}

sub get_objects_ui {
    my $tb = $r->ToolBar(
                     buttons => [[ 
                     {type=>'image',icon=>'delete', name=>'remove', text=>text('revoke')},
                                ]],
                            );
    my $objects_ui = $r->Grid(
                            name => 'obj_grid',
                            data => \&get_objects ,
                            page => 0,
                            selectable=>1,
                            page_size=>10,
                            title=>text('publ_object'),
                            columns=>3,
                            toolbar => $tb,
                            width=>500,
                            action=>"publication.cgi?id=$id",
                            ddata => [
                                        { 
                                          width=>100,
                                          description=>text('description'),
                                          extra=>'nowrap',
                                        }, 
                                        { 
                                          width=>200,
                                          description=>text('path'),
                                        }, 
                                        { 
                                          width=>200,
                                          description=>text('modified'),
                                        },
                                     ],
                           );   

    return $objects_ui;
}


sub get_objects {
    my ($s, $e) = @_;
    unless ( $r->{objects} and ref($r->{objects}) eq "ARRAY" ) {
        $r->{objects} = [];
        # read selected rows
        my %selected;
        my $data = {};
        if ($r->{objects_ui}->read_input($REQUEST)) {
            $data = $r->{objects_ui}->result_data();
            if ( exists($data->{selected_row}) and defined($data->{selected_row}) ) {
                %selected = map { $_=>1 }
                            ref($data->{selected_row}) eq 'ARRAY' 
                                ? @{$data->{selected_row}}
                                : ($data->{selected_row});
            }
        }    
        # get hosts list and call actions
        my @objects;
        my $i = 0;
	foreach my $obj_type ( qw(file list map) ) {
	    if ( $REQUEST->query($obj_type) ) {
		my $objs = $REQUEST->query($obj_type);
		$objs = [$objs] unless ref($objs) eq "ARRAY";
		foreach (@$objs) {
		    my $desc = $REQUEST->query('file_desc') || $_;
		    $p->add_file($_, obj_type=>$obj_type,
				 description=>$desc);
		}
	    }
	    if ( my $err_msg = $p->error ) {
		$r->raise_error($err_msg);
	    }
	    my $o = $p->files(obj_type=>$obj_type);
	    foreach ( keys %$o ) {
		my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime);
		if ( -f $PREFIX."/".$o->{$_}{path} ) {
		    ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime) = stat($PREFIX."/".$o->{$_}{path});
		    my($sec,$min,$hour,$mday,$mon,$year) = localtime($mtime);
		    $mtime = sprintf("%04d-%02d-%02d %02d:%02d:%02d%s",
				     $year+1900, $mon+1, $mday, $hour, $min, $sec, "");
		} else {
		    $mtime = "No file present";
		}
		if ( $selected{$i} and $data->{remove} ) {
		    $p->revoke_file($_, obj_type=>$obj_type);
		}
		else {
		    push @objects, [ $o->{$_}{description}, $o->{$_}{path}, $mtime ];
		}
		$i++;
	    }
	}
        $r->{objects} = \@objects;
    }
    if ( defined $s ) {
        return $r->{objects};
    }
    else {
        return scalar @{$r->{objects}};
    }
    
}
 
sub hosts_tb {
    my @buttons;
    if ( get_subscribers() ) {
        push @buttons, ( {type=>'text', name=>'sync', text=>text('synchronize')},
                         {type=>'text', name=>'remove', text=>text('unsubscribe')}, );
    }
    
    my $combo = hosts_combo();
    if ( $combo ) {
        push @buttons, {type=>'custom', text=>$combo};
        push @buttons, {type=>'text', name=>'add', text=>text('subscribe')};
    }

    my $tb = $r->ToolBar(buttons => [\@buttons]);
    return $tb;
}

sub hosts_combo {
    my $html;
    my $h = get_hosts();
    my $s = get_subscribers(1);
    my %s = map {$_->[0]=>1} @$s;
    foreach ( keys %{$h} ) {
        next if is_self($h->{$_});
        $html .= "<option value='$_'>$_</option>" unless $s{$_};
    }
    return undef unless $html;
    return "<select name=add_host><option value=''>(".text('select_host').")</option>$html</select>";
}

sub get_hosts_ui {
    my $hosts_ui = $r->Grid(
                            name => 'hosts_grid',
                            page => 0,
                            data => \&get_subscribers,
                            selectable=>1,
                            page_size=>10,
                            title=>text('subscribers'),
                            columns=>3,
                            toolbar => \&hosts_tb,
                            width=>500,
                            action=>"publication.cgi?id=$id",
                            ddata => [
                                        { 
                                          width=>100,
                                          description=>text('host_name_alias'),
                                          extra=>' nowrap="yes" ',
                                        }, 
                                        { 
                                          width=>200,
                                          description=>text('last_updated'),
                                        }, 
                                        { 
                                          width=>200,
                                          description=>text('status'),
                                        },
                                     ],
                           );   
    $hosts_ui->read_input($REQUEST);
    return $hosts_ui;
}


sub get_subscribers {
    my ($s, $e) = @_;
    unless ( $r->{hosts} and ref($r->{hosts}) eq "ARRAY" ) {
        $r->{hosts} = [];
        # read selected rows
        my %selected;
        my $data = $r->{hosts_ui}->result_data();
        if ( exists($data->{selected_row}) and defined($data->{selected_row}) ) {
            %selected = map { $_=>1 }
                        ref($data->{selected_row}) eq 'ARRAY' 
                            ? @{$data->{selected_row}}
                            : ($data->{selected_row});
        }
        
        # get hosts list and call actions
        my $h;
        if ( $data->{sync} ) {
            $h = $p->subscribers(verify=>1);
            my @to_sync;
            my $i = 0;
            foreach ( keys %$h ) {
                push @to_sync, $_ if $selected{$i};
                $i++;
            }
            my $s = $p->sync(@to_sync); # publication sync call
            foreach ( keys %{$s} ) {
                $h->{$_}{status} = $s->{$_}{status};
            }
        }
        else {
            $h = $p->subscribers(verify=>1);
        }
        my $i = 0;
        foreach ( keys %$h ) {
            if ( $selected{$i} and $data->{remove} ) {
                $p->unsubscribe($_);
            }
            else {
                my $newest = 0;
                my $outofsync = "<br/>&nbsp;" . 
                    join("<br/>&nbsp;", keys %{$h->{$_}{off_sync}});

                foreach (values %{$h->{$_}{files}}) {
                    $newest = $_->{modified} if $_->{modified} > $newest;
                }
                my $last_updated = "-";
                if ($h->{$_}{last_updated}) {
                    my($sec,$min,$hour,$mday,$mon,$year) = localtime($h->{$_}{last_updated});
                    $last_updated = sprintf("%04d-%02d-%02d %02d:%02d:%02d%s",
                                      $year+1900, $mon+1, $mday, $hour, $min, $sec, "");
                } elsif ( $newest ) {
                    my($sec,$min,$hour,$mday,$mon,$year) = localtime($newest);
                    $last_updated = sprintf("%04d-%02d-%02d %02d:%02d:%02d%s",
					    $year+1900, $mon+1, $mday, $hour, $min, $sec, "");
		}
                push @{$r->{hosts}}, [ $_, $last_updated, $h->{$_}{status} . $outofsync ];
            }
            $i++; 
        }
    }
    if ( defined $s ) {
        return $r->{hosts};
    }
    else {
        return scalar @{$r->{hosts}};
    }
}
